$BASE 10T
$PAGEWIDTH 132

;**************************************************************************
;                         "Micro908 AA908"
;                          Antenna Analyst
;
;                        George Heron, N2APB
;                        2419 Feather Mae Ct.
;                        Forest Hill, MD  21050
;                        email: n2apb@amsat.org
;
; -------------------------------
; Contributors to the sourcecode:
;
;   WARC Band Coverage, Link908 & data sending:
;       Michael L. Hasenfratz, WA6FXT
;       131 Columbia Rd.
;       Thousand Oaks, Ca. 91360
;       email: wa6fxt@arrl.net
;
; SEEPROM Driver for non-volatile scan data store & storing multiple program
;       Steve Holton, N1NB
;
;---------------------------------
;
; Revision History:
; ----------------
; 1.0      10/4/2003      Initial prototype release before Pacificon QRP Forum
; 1.1      1/18/2004      Fix for dial granularity
; 1.3      2/28/2004      (1) Added better use of dial for submenu selection, (2) Fixed display intermittency
; 1.4      3/31/2004      (1) Added SWR calculations in Manual mode, (2) Added Check4Min SWR during
;                         Scan, (3) Changed stack to $03FF, (4) Added Custom Settings: Start, End, Step;
;                         (5) Added EEPROM save/restore for user settings, (6) Changed Scan end-checking
;                         to use Scan_End variable, (7) Added "View User Settings" to Config menu
; 1.5      4/14/2004      (1) New way to do piezo tones, (2) Tied tones in with SWR when in Manual.
; 1.6      5/11/2004      User interface changes: (1) Continuous SWR & Z measurements in Manual,
;                         (2) User dumped into Manual at end of Scan, (3) Config renamed to Setup,
;                         (4) Menu messages trimmed & reordered, (5) Cal mode removed from Mode menu,
;                         (6) Dynamic menu entry for Setup, Band and Scan now avail from within Manual,
;                         (7) Resisance measurements now made, (8) SWR fixes yield right value for R>50.
; 1.7      5/13/2004      Resistance measurements cleaned up.
; 1.7e     6/04/2004      Added ability for HC908 Daughtercard to programmatically
;                         download/flash a new application with command from the
;                         application code, in cooperation with custom "m908
;                         Loader" program on PC communicating with HC908 card
;                         over serial link.
; 2.0      6/22/2004      (1) Implemented fourth set of reflectometer detectors and buffer amps for "Va" voltage
;                         in order to calculate better Z. Works great! (2) Added Debug Mode that sends intermediate
;                         computations to the serial terminal.
; 3.0k     9/02/2004      (1) Added Calibration, (2) enter to Set DDS limits at prior settings,
;                         (3) added 100 kHz step size, (4) used greater precision computations,
;                         (4) added busy LED during Save Settings, (5) added "exit=<Setup>" in Set Start/End,
;                         (5) added DEBUG mode to dump data block, (6) added "Return to Default" Config cmnd,
;                         (7) fixed Scan hanging bug (properly set Scan_End(0:7))
;                         (8) brought Debug Monitor command up to main level in Config menu,
;                         (9) eliminated Diags module (not needed)
; 3.0k6    10/18/2004     (1) Changed to using ERA-3 with different gain resistors.  Getting rid of MAV-11 solved
;                         high-end signal level rolloff
;                         (2) Changed to "back_to_basics" simplified calibration, adjusting Vr to make SWR=5.4
;                         with 270-ohm load.  Best performing results ever!!
;                         (3) Added LCD backlight control circuit (and software timers, Setup menus, etc.)
;                         (4) Fixed bug in calculating R (was improperly scaling large denominator)
; 4.0a    11/15/2004      Started measuring and adjusting for A/D offset voltages.  Improved accuracy a bit.
;                         Now handling remainder of correction factor generation and use.  Yields best results yet!
;                         Fixed cursor adjust too far to left.
;                         Straightened out Lite control and EEPROM save/restore
;                         Moved cal data to EEPROM Page 1.
;                         Fixed power-on pushbutton so timing not critical on release.
;                         Added "Stopped" display at powerdown.
;                         Made the Default Settings option more sensible and usable.
;                         Determine & display reactance sign
;                         Added modes for Inductance, Capacitance and VFO
; 4.0b    11/28/2004      Removed experimental code for Power and LCD Backlight Control.  (Caused probs in v4.0a)
;                         Removed Config "Software Update" option (used with m908 Loader pgm on PC) ... too error prone.
; 4.0c    12/05/2004      Mode pushbutton user interface change
;                         Calibration point changed to 10 MHz
; 4.1a    01/23/2005      Band selection increased to include WARC bands.  (WA6FXT)
;                         Added Link908 interface (WA6FXT)
; 4.1c    02/12/2005      Corrected stack bugs in Get_Dial_Selection and Get_YN routines
;                         Corrected portA bug in LCD driver
;                         Corrected Reactance Sign feature (in x_sign.asm)
;                         Merged WARC Band sppt and raw Link908 data transmit to mainline code
;                         Added Config options for scan data (no save, send to PC, send to SEEPROM)
;                         Added placeholders for SEEPROM code (search .LST file for "INSERT SEEPROM USAGE HERE")
; 4.1d    02/13/2005      Merged SEEPROM development with mainline code, with SEEPROM test function hooked to PB5
; 4.1e    02/17/2005      Added R & X send options for Link908 (WA6FXT)
;                         Corrected stack error in scan.asm (send_n_mem:) (WA6FXT)
;                         Corrected bug in link.asm (CustomBand) (WA6FXT)
; 4.1     02/27/2005      Production code = supports reactance sign and Link908
;
;**************************************************************************
; Copyright 2005 by G. Heron, N2APB.  All rights reserved.
;**************************************************************************

        include "ab_regs.asm"            ;MC68HC908AB32 register definitions
        include "macros.asm"             ;Macro instruction definition
        include "local.asm"              ;Local system variables

;**************************************************************************
;  EQUATES
;**************************************************************************

RAMStart     equ  $0053         ;User App RAM is from 53 to 3FF. (HCmon RAM from 400-44F)
EEPROM       equ  $800          ;EEPROM mem is 800-9FF (512 bytes)
RomStart     equ  $8000         ;Flash mem is 8000 to FFFF (32 MB)

                                ;Port A pin usage
KbdData      equ  7               ;Keyboard data line
DSP_RST      equ  6               ;DSPx Reset
DSP_RXD      equ  5               ;DSPx RxD data line
DSP_TXD      equ  4               ;DSPx TxD data line
LCD_e        equ  3               ;LCD enable = bit 3 of porta
LCD_rw       equ  2               ;LCD read/write. 0=write, 1=read
LCD_rs       equ  1               ;LCD register select. 0=instruction, 1=data
KbdClk       equ  0               ;Keyboard clock line

                                ;Port B pin usage
CHAN_Va      equ  4               ;Reflectometer Va voltage
CHAN_BATT    equ  3               ;Battery voltage
CHAN_Vf      equ  2               ;Reflectometer Forward Voltage
CHAN_Vz      equ  1               ;Reflectometer Impedance Voltage
CHAN_Vr      equ  0               ;Reflectometer Reflected Voltage

                                ;Port C Usage
Keyline      equ  5               ;Keyline
BusyLED      equ  4               ;Busy LED
Dah          equ  3               ;Dah paddle input
Dit          equ  1               ;Dit paddle input

                                ;Port D pin usage  (LCD data lines)

                                ;Port E Usage
SEE_SCK      equ  7               ;SEEPROM SCK
SEE_SI       equ  6               ;SEEPROM SI
SEE_SO       equ  5               ;SEEPROM SO
SEE_CS       equ  4               ;SEEPROM CS
Piezo        equ  2               ;Piezo sounder
RxData       equ  1               ;serial port receive
TxData       equ  0               ;serial port transmit

                                ;Port F pin usage
Mon_Jumper   equ  7               ;Monitor jumper
HeartbeatLED equ  6               ;Heartbeat LED
ENC_A        equ  5               ;Encoder-A pin
ENC_B        equ  4               ;Encoder-B pin
ENC_pb       equ  3               ;Encoder-Pushbutton pin
DDS_load     equ  2               ;DDS load pin
DDS_clk      equ  1               ;DDS clock pin
DDS_data     equ  0               ;DDS data pin

                                 ;Port H Usage
ModePB       equ  1               ;Push Button 2
BandPB       equ  0               ;Push Button 3

                                 ;Port G Usage
ScanPB       equ  2               ;Push Button 4
ConfigPB     equ  1               ;Push Button 5
SleepPB      equ  0               ;Push Button 6

                                ;Measurement Modes
Impedance_Mode       equ  0
VFO_Mode             equ  1
Link908_Mode         equ  2
Capacitance_Mode     equ  3
Inductance_Mode      equ  4

                                ;Data Mode ... how to handle data during scan:
NoData               equ  0     ;  Save data from Scan
Data2Mem             equ  1     ;  Save data from Scan to SEEPROM
SendDuringScan       equ  2     ;  Send data during Scan
SendFromMem          equ  3     ;  Send data from SEEPROM *now* (this is actually a command from Config)

;**************************************************************************
;  PAGE 0 VARIABLE MEMORY ALLOCATION
;**************************************************************************

    org RamStart                ;$0053

_hbufpos     rmb  2             ;buffer position temp storage

sens_sav     ds   1             ;save for entered encoder Sensitivity value
sens_cw      ds   1             ;CW sensitivity control
sens_ccw     ds   1             ;CCW sensitivity control

DDS_active   ds   1             ;flag used in Process_Dial to include DDS ctl & SWR calcs

                                ;1ms timeout counters used in Timer ISR
Timeout1     ds   2             ;16-bit timeout counter used for HeartBeat LED
Timeout2     ds   2             ;16-bit timeout counter

LCD_timer    ds   1             ;counter for timeout check in LCD busy_check
LCD_read     ds   1             ;temp store for status read in LCD busy_check
LCD_char     ds   1             ;temp for LCD data written (cmnd2LCD & data2LCD)
temp1        ds   1             ;temp store used in write_LCD and DDS mpy + add routine
temp2        ds   1

temp         ds   1             ;temp used in EEPROM driver

enc_new      ds   1             ;new reading of encoder
enc_old      ds   1             ;last reading of encoder
enc_temp     ds   1             ;temp location for encoder bits
A_flag       ds   1             ;enc line A hi/lo status
B_flag       ds   1             ;enc line B hi/lo status

mult_count   ds   1             ;Used in calc_dds_word
bit_count    ds   1             ;   "
byte2send    ds   1             ;
byte_count   ds   1             ;DDS byte counter

DDS_w4       ds   1             ;DDS programming register W4 (freq-b7  ...  freq-b0) [LSB]
DDS_w3       ds   1             ;DDS programming register W3 (freq-b15 ...  freq-b8)
DDS_w2       ds   1             ;DDS programming register W2 (freq-b23 ...  freq-b16)
DDS_w1       ds   1             ;DDS programming register W1 (freq-b31 ...  freq-b24)
DDS_w0       ds   1             ;DDS programming register W0 (phase-b4 ...  control-0) [MSB]

LCD_dat      ds   7             ;LCD data display for frequency: XX,XXX.XX  [LCD_dat+0 is MSD]
Scan_Start   ds   7             ;Scan START freq, placed into LCD_dat in BAND processing, used in Scan
Scan_End     ds   7             ;Scan END freq, placed into LCD_dat in BAND processing, used in Scan

;-----------------------------------------------------------------------------
; START of "Page 0 data" saved to & restored from EEPROM
;      Freq Dial display array:
;            LCD_Dat+0, LCD_Dat+1, LCD_Dat+2, LCD_Dat+3, LCD_Dat+4, LCD_Dat+5, LCD_Dat+6
;              10 MHz     1 MHz     100 kHz     10 kHz     1 kHz     100 Hz      10 Hz
;     StepPtr =  00        01         02         03         04         05          06
;
NV_STRT      equ  *		;start of Non Volitile (EEPROM) variables
Custom_Start ds   7             ;holds Custom Start frequency, as set in Config/Start_Freq
Custom_End   ds   7             ;holds Custom End frequency, as set in Config/End_Freq
StepPtr      ds   2             ;holds step increment, used in Scan routine, set in Config/ScanStepSize
Tone_Flag    ds   1             ;flag turns tone on during manual scan
DataMode     ds   1             ;indicates how to handle data during scan:
                                ;          0 = NoData = Save data from Scan
                                ;          1 = Data2Mem = Save data from Scan to SEEPROM
                                ;          2 = SendDuringScan = Send data during Scan
                                ;          3 = SendFromMem = Send data from SEEPROM *now*
NV_SIZ       equ  *-NV_STRT	;Size of NV data
;
; END of "Page 0" data saved/restored from EEPROM
;------------------------------------------------------------------------------

op2          ds   1             ;operand 2 for DDS multiply+add calculation

start_index  ds   2
cursor_index ds   2             ;variable for LCD digit address used in setting cursor
step_count   ds   2             ;variable for counting scan steps

;---------------------keep the next 8 bytes contiguous for SPI usage  SMH
;---------------------    AND in this order for SPI usage             SMH
;            ---------  these  2 bytes used only by SPI and wait500ms SMH
SPI_Args     equ  *             ; 2 x 16 bit argument list            SMH
SPI_RamAddr  equ  *             ;16 bit RAM addr                      SMH
SPI_toAddr   equ  *             ;

count_lo     ds   1
count_hi     ds   1
;               ---------  these  6 bytes used only by SPI and EEPROM SMH
SPI_XferArgs equ  *
SPI_Offset   equ  *             ;16 bit SEEPROM offset                SMH

to_addr      ds   2             ;index pointer storage locs

SPI_Count    equ  *             ; SPI 16 bit byte count               SMH
SPI_CountHi  equ  *             ; SPI  count HI byte                  SMH
SPI_CountLo  equ  *+1           ; SPI  count LO byte                  SMH

from_addr    ds   2

SPI_Ptr      equ  *             ;16 bit buffer pointer                SMH
SPI_Count2   equ  *             ; SPI 16 bit byte count2              SMH

EEbyte_count ds   1             ;EEPROM byte counter
EEline_count ds   1             ;EEPROM line counter
;-----------------------end contiguous -------------------            SMH

msg_cntr     ds   1             ;counter used as index into Msg Table
msg_pointer  ds   2             ;storage for message pointer
CmndTabPntr  ds   2
CmndTabStart ds   2
MsgTabStart  ds   2
MsgTabPntr   ds   2

intacc1      ds   4             ;32-bit integer accumulator #1
intacc2      ds   4             ;32-bit integer accumulator #2

z            ds   4             ;32-bit integer store for Z
z_squared    ds   4             ;32-bit integer store for Z^2
r_temp       ds   4             ;32-bit integer store for (2500+Z^2) in Calc_R
SWR          ds   4             ;32-bit integer store for SWR
SWRx100      ds   4             ;32-bit integer store for SWRx100
SWR_temp     ds   4             ;32-bit integer store for 50 * (SWR^2 +1)
R            ds   4             ;32-bit integer store for R

spval        ds   2             ;storage for stack pointer value

WW           ds   1             ;SWR calculation whole number
DD           ds   1             ;SWR calculation decimal remainder

WW_min       ds   1             ;Min SWR - whole number
DD_min       ds   1             ;Min SWR - decimal number
F_min        ds   7             ;Min frequency (LCD_dat digits)

bcd100000    ds   1
bcd10000     ds   1
bcd1000      ds   1             ;results of Bin2BCD
bcd100       ds   1
bcd10        ds   1
bcd1         ds   1

sqrt         ds   2             ;used in imath.asm

Reactance    ds   2             ;binary Reactance value
Current_React ds   2             ;Current Reactance value
Freq         ds   2             ;binary frequency (10M+1M+100K+10K+1K) from LCD_Dat(0:6) Freq registers
lchar        ds   1             ;Link char save area

;**************************************************************************
;  EXTENDED VARIABLE MEMORY ALLOCATION (i.e. addr $0100 and upward)
;**************************************************************************

Vf           ds   1             ;ADC reading for Vf
Vr           ds   1             ;ADC reading for Vr
Vz           ds   1             ;ADC reading for Vz
Va           ds   1             ;ADC reading for Va
EE_dirty     ds   1             ;dirty flag used in saving Config settings
enc_units    ds   1             ;variables used to display encoder counter
enc_tens     ds   1             ;  in PwrDwn and LiteOff option
Tone         ds   1             ;8-bit value used for piezo tone (5=500Hz...10=1kHz...100=10kHz)
debug_flag   ds   1             ;set=1 to see debug print statements to terminal
iCalc_flag   ds   1             ;flag set during dial idle times to perform Calc/Display in heartbeat ISR
Va_acc       ds   4
Vfx100       ds   4             ;accumulators (x100) for Vf, Vr, Vz and Va ADC readings
Vr_acc       ds   4
Vz_acc       ds   4
Z_maxed      ds   1             ;flag indicates when Z computation exceeds 600

temp1_acc    ds   4             ;temporary accumulators
temp2_acc    ds   4
sign         ds   1             ;sign of the reactance.  Either '+' (inductive) or '-' (capacitive)
No_Disp_Flag ds   1             ;set flag to have a silent computation of SWR, R and X (used in X_sign.asm)
FreqSave     ds   7             ;save locations for frequency dial LCD_Dat(0:6)
SWR_Disp_Flag ds  1             ;allows display of SWR label in Process_Dial (Mode.asm)
R_Disp_Flag  ds   1             ;allows display of R in Calcs
X_Disp_Flag  ds   1             ;allows display of X in Calcs
Mode_nr      ds   1             ;Mode number (0=imp, 1=VFO, 2=cap, 3=ind)

                                ; SMH SPI Contiguous 256 byte buffer
                                ; MUST be contiguous SMH
SPI_buffer1 ds 128              ; SMH 1st 1/2 of SPI Contiguous 256 byte buffer
SPI_buffer2 ds 128              ; SMH 2nd 1/2 of SPI Contiguous 256 byte buffer

;;---**** Temporary storage for SEEPROM Tests remove when done SMH
WR_First_Initial  ds 1             ; initial pattern byte
WR_First_Byte     ds  1
;;---**** end Temporary storage for SEEPROM Tests remove when done SMH

;-----------------------------------------------------------------------------
; START of "Page 1 data" saved to & restored from EEPROM (calibration data)

Vf_offset    ds   1             ;op amp offsets with no signal present
Va_offset    ds   1
Vr_offset    ds   1
Vz_offset    ds   1
Va_corrfact  ds   2             ;Va correction factor ... first byte + remainder byte
Vr_corrfact  ds   2             ;Vr correction factor ... first byte + remainder byte
Vz_corrfact  ds   2             ;Vz correction factor ... first byte + remainder byte
;
; END of "Page 1 data" saved/restored from EEPROM (calibration data)
;------------------------------------------------------------------------------


    org $03FF
;      (Stack starts here and works down toward $0053)

    org $0400
;      ($0400 - $04FF used by HCmon)

    org $040A
_inbuf       rmb  RXBLEN        ;common "_inbuf" buffer with HCmon

;**************************************************************************
;   Vector Jump Table (located at start of User Area = 8000)
;**************************************************************************

    org RomStart

_user_adc         jmp    Dummy_ISR      ; Insert jump to ADC vector code
_user_keyboard    jmp    Dummy_ISR      ; Insert jump to Keyboard vector code
_user_scitx       jmp    Dummy_ISR      ; Insert jump to SCI transmit vector code
_user_scirx       jmp    Dummy_ISR      ; Insert jump to SCI receive vector code
_user_scierr      jmp    Dummy_ISR      ; Insert jump to SCI error vector code
_user_timbch3     jmp    Dummy_ISR      ; Insert jump to TIMB Channel 3 Vector code
_user_timbch2     jmp    Dummy_ISR      ; Insert jump to TIMB Channel 2 Vector code
_user_spitx       jmp    Dummy_ISR      ; Insert jump to SPI transmit vector code
_user_spirx       jmp    Dummy_ISR      ; Insert jump to SPI receive vector code
_user_timb_of     jmp    Dummy_ISR      ; Insert jump to TIMB Overflow Vector code
_user_timb_ch1    jmp    Dummy_ISR      ; Insert jump to TIMB Channel 1 Vector code
_user_timb_ch0    jmp    Dummy_ISR      ; Insert jump to TIMB Channel 0 Vector code
_user_tima_of     jmp    Dummy_ISR      ; Insert jump to TIMA Overflow Vector code
_user_tima_ch3    jmp    Dummy_ISR      ; Insert jump to TIMA Channel 3 Vector code
_user_tima_ch2    jmp    Dummy_ISR      ; Insert jump to TIMA Channel 2 Vector code
_user_tima_ch1    jmp    T_ISR_A1       ; Insert jump to TIMA Channel 1 Vector code
_user_tima_ch0    jmp    T_ISR_A0       ; Insert jump to TIMA Channel 0 Vector code
_user_pit         jmp    Dummy_ISR      ; Insert jump to PIT Vector code
_user_pll         jmp    Dummy_ISR      ; Insert jump to PLL Vector code
_user_irq         jmp    Dummy_ISR      ; Insert jump to ~IRQ1 Vector code

_HCmon_LCD_disp   jmp    HCmon_LCD_disp ; vector (at $803C) for HCmon
                                        ; to display message to LCD

;**************************************************************************
;**********************      USER APP ENTRY POINT     *********************
;**************************************************************************
;        This is the point ($8040) where User Code starts executing       *
;        after coming from the Monitor "X" command (eXecute User App)     *
;        or after board gets a RESET (when Monitor jumper is off)         *
;**************************************************************************

        org     $8040

Start:
        sei                            ;disable all interrupts
        ldhx    #$03FF                 ;initialize
        txs                            ;  the stack pointer

Restart:
        jsr     Init_all               ;initialize system variables and devices

        cli                            ;allow interrupts to happen

;**************************************************************************
;**************************************************************************
;*****************      USER APPPLICATION MAINLINE      *******************
;**************************************************************************
;**************************************************************************

User_Main:
        mov     #3,timeout1            ;I'm not sure why this is needed (but it is!)
        jsr     LCD_init
        sta     copctl                 ;clear the COP counter
        lda     #100
        jsr     delay

             ;display battery voltage
        jsr     LCD_Clear
        jsr     DispBatt1
        jsr     wait500ms

startmain:
        tone_off                        ;turn tone off while in Main menu
st7:    jsr     LCD_Clear
        ldhx    #AA1_msg
st5:    jsr     disp_msg                ;display banner on 1st line
        jsr     wait500ms
        jmp     Impedance               ;go right into Impedance Mode (in Mode.asm module)

AA1_msg: fcb     '  AA908 4.1',0
AA2_msg: fcb     'Select Function',0

;**************************************************************************
;***************      HCmon_LCD_disp        *******************************
;*     Routine called from HCmon to display message to LCD                *
;**************************************************************************

HCmon_LCD_disp:
        rts

;**************************************************************************
;*****************       DELAY ROUTINES     *******************************
;**************************************************************************

;--------------------------------------------------------------------------
;   Delay -- This subroutine performs a simple software delay loop based upon
;            the value passed in ACC. The following timing calculation applies:
;               delay = ((ACC * 74) + 12) (tcyc)

delay:
        psha                           ;[2] save delay parameter temporarily
delay1:
        lda #22T                       ;[2] initialize 5us loop counter
                                       ;     (repeat for timing)
delay2:
        dbnza delay2                   ;[3] decrement inner delay loop counter
        dbnz 1,sp,delay1               ;[6] decrement outer delay loop counter
        pula                           ;[2] deallocate local variable
        rts                            ;[4] return

;-----------------------------------------------------
; Wait500ms -- Waits here for (10 x 50 x 1ms) = 500ms

wait500ms:
        lda     #10
        sta     count_hi
w500m1: lda     #$50
        sta     count_lo
w500m2: jsr     wait1ms
        dbnz    count_lo,w500m2
        dbnz    count_hi,w500m1
        rts

;-------------------------------------------------------
; Wait100ms-- Waits here for 100ms

wait100ms:
        lda     #100
w100a:  psha
        bsr     wait1ms
        pula
        dbnza   w100a
        rts

;--------------------------------------------------------
; Wait64ms-- Waits here for 64ms

wait64ms:
        lda     #64
w64a:   psha
        bsr     wait1ms
        pula
        dbnza   w64a
        rts

;---------------------------------------------------------
; Wait50ms-- Waits here for 50ms

wait50ms:
        mov     #50,timeout1
w50a:   tst     timeout1
        bne     w50a
        rts

;----------------------------------------------------------
; Wait32ms-- Waits here for 32ms

wait32ms:
        lda     #32
w32a:   psha
        bsr     wait1ms
        pula
        dbnza   w32a
        rts

;-----------------------------------------------------------
; Wait64ms-- Waits here for 16ms

wait16ms:
        lda     #16
w16a:   psha
        bsr     wait1ms
        pula
        dbnza   w16a
        rts

;------------------------------------------------------------
; Wait10ms-- Waits here for 10ms

Wait10ms:
        lda     #10
w10a:   psha
        bsr     wait1ms
        pula
        dbnza   w10a
        rts

;------------------------------------------------------------
; Wait1ms-- Waits here for 1ms

wait1ms:
        lda     #10
w1msa:  psha
        bsr     wait100us
        pula
        dbnza   w1msa
        rts

;-------------------------------------------------------------
; Wait100us -- Waits here for 100us
;              1 cycle = .11454545 US
;              (137 x (5 x .11454545)) + (6 x .11454545)
;              (174 x .5727) + .687 = 100 us

wait100us:
        lda     #174       ; [2]
w100u:  deca               ; [1]
        nop                ; [1]
        bne     w100u      ; [3]
        rts                ; [4]

;--------------------------------------------------------------
; Wait1us -- Waits here for 1us

wait1us:
        lda     us1
        dbnza   *
        rts

;*************************************************************
;*****************     TIMER MODULE    ***********************
;*************************************************************

;-------------------------------------------------------------
; Init_Timer_Module - Turns on timer A channel 1 for an Output
;              Compare in 1ms. Creates an interrupt
;              vector to the Timer ISR (T_ISR_A1), where Timeout1,
;              Timeout2 and Timeout3 are decremeted. If a timer
;              reaches 0, then a user-specified operation is
;              done.  Thus, the user can either set a timer in his
;              code and sit there waiting on it to become 0, or
;              he may place the timeout action routine right here
;              in the Timer Handlers.  We use the latter technique
;              for the "heartbeat" LED blink. We set Timeout1 to
;              255 in the Timer1 Handler below, and when 255 ticks
;              of the 1ms timer interrupt occur, we toggle the state
;              of the heartbeat LED and reset Timeout1 to occur again
;              in another 255 ticks.
;---------------------------------------------------------------

Init_Timer_Module:
       mov     #$36,tasc           ; Timer A cleared + stopped. Clicks every 64 bus cycles
       mov     #0,tach1H           ; Set Output Compare to happen in 136 clicks (1 ms)
       mov     #136,tach1L         ;   after we start the timer.
       mov     #0,tach0H           ; Set Output Compare to happen in 65 clicks (0.5 ms)
       mov     #65,tach0L          ;   after we start the timer.
       mov     #$54,tasc1          ; Set Timer A Chan 1 for Output Compare operation
;       mov     #$54,tasc0          ; Set Timer A Chan 0 for Output Compare operation.
       mov     #$06,tasc           ; Start the timer
       rts

;*******************************************************************
; T_ISR_A0 - Timer Interrupt Service Routine for Timer A Chan 2
;               272 clicks = 2.0ms  period =    250 Hz tone
;               136 clicks = 1.0ms  period =    500 Hz tone
;                65 clicks = 0.5ms  period =  1,000 Hz tone
;                13 clicks = 0.1ms  period =  5,000 Hz tone
;                07 clicks = 0.05ms period = 10,000 Hz tone
;*******************************************************************

T_ISR_A0:
       pshh
       pshx
       psha

       bclr    7,tasc0               ;clear OC flag
       ldhx    tach0h
       txa
       add     Tone
       tax
       bcc     ta01
       inch
ta01:  sthx    tach0h                ;setup next interrupt in (h:x) clicks

       pula
       pulx
       pulh
       rti

;*******************************************************************
; T_ISR_A1 - 1ms Timer Interrupt Service Routine for Timer A Chan 1
;*******************************************************************

T_ISR_A1:
       pshh
       pshx
       psha

       bclr    7,tasc1              ;clear OC flag
       ldhx    tach1h
       aix     #127                 ;setup next interrupt in (127+9) clicks (1ms)
       aix     #9
       sthx    tach1h

Check_t1:                           ;Timer 1 = heartbeat timer
       ldhx    timeout1
       aix     #-1                  ;decrement timer 1
       sthx    timeout1
       bne     check_t2             ;not finished counting down, so go check next timer
       jsr     t1_handler           ;timer 1 exhausted, go execute Timeout 1 handler

Check_t2:
       ldhx    timeout2             ;Timer 2 active?
       cphx    #0
       beq     done_A1              ;no, just exit
       aix     #-1                  ;Timeout2 is active. Decrement it.
       sthx    timeout2

done_A1:
       pula
       pulx
       pulh
       rti

;---------------------------------------------------------------
; Timer A Chan 1 (TACH1) Timeout Handlers
;                    All the user has to do is set one of the
;                    timeout variables to desired delay n (1-255)
;                    and the timeout handler will be executed
;                    in n milliseconds. Setting the timeout
;                    variable from within the handler will
;                    cause a periodic timeout as shown in
;                    timeout 1.  (This is used to driver the
;                    Heartbeat LED.)
;---------------------------------------------------------------

t1_handler:
       lda     iCalc_flag           ;is "interrupt Calc flag" on? (Set in Process_Dial during dial static times)
       beq     t1b                  ;no, continue
       jsr     Do_Calcs             ;yes, calculate/display SWR, Z, R, X (and L or C, if in those modes)
       lda     Mode_nr
       cmp     #Impedance_Mode      ;are we in impedance mode?
       bne     t1b                  ;no, just continue
       jsr     Disp_Sign            ;determine & display sign of reactance for Impedance Mode, during
                                    ;  dial "quiescent" times here in ISR
t1b:   ldhx    #255
       sthx    timeout1             ;reset timer 1 to 255 ms
       brclr   6,portF,t1a          ;and toggle heartbeat LED
       bclr    6,portF
       rts
t1a:   bset    6,portF
       rts

t2_handler:                         ;used for the Piezo beep duration
       nop
       rts

;**************************************************************
; DUMMY_ISR - Dummy Interrupt Service Routine.
;             Just does a return from interrupt.
;**************************************************************
dummy_isr:
       nop
       rti           ; return

;**************************************************************
;  INCLUDED MODULES
;**************************************************************

      include "mode.asm"
      include "config_m.asm"
      include "calib.asm'
      include "scan.asm"
      include "bands.asm"
      include "drivers_m.asm"
      include "math.asm"
      include "debug.asm"
      include "calcs.asm"
      include "init_m.asm"
      include "x_sign.asm"
      include "link.asm"
      include "SPI_functions.asm"    ; SEEPROM functions  SMH
      include "SPI_FileXfer.asm"     ; SEEPROM<->PC data transfer functions SMH

;;;----*** Temporary remove when done SMH
      include "SEEPROM-Test-loop.asm"    ; SEEPROM test routine  SMH
;;;----*** end Temporary remove when done SMH

;**************************************************************
; UTILS - Utilities and routines shared from HCmon
;         Just link to routines in HCmon
;         (But remember to adjust jump addresses if HCmon
;         ever changes!)
;         An alternative to linking like this is to include the line:
;             include "../utils.asm"    ; but this approach replicates code!
;**************************************************************

_sp_gets:     jmp   $F4DB
_gets:        jmp   $F4EE
_ahex2b:      jmp   $F53F
_a2nibhex:    jmp   $F54D
_dblchar:     jmp   $F556
_nib_to_asc:  jmp   $F561
_comma_sp:    jmp   $F56C
_pr_letter_x: jmp   $F577
_pr_pound:    jmp   $F57C
_pr_comma:    jmp   $F581
_crlf:        jmp   $F586
_byte_sp:     jmp   $F58F
_space:       jmp   $F592
_pr_nibble:   jmp   $F599
_putch:       jmp   $F59C
_pwordx:      jmp   $F5A5
_pword:       jmp   $F5AD
_pbyte_sp:    jmp   $F5B9
_pbytex:      jmp   $F5BD
_pbyte:       jmp   $F5BE
_ischar:      jmp   $F5CD
_getchne:     jmp   $F5D7
_getch:       jmp   $F5E1
_do_again:    jmp   $F5F6
_puts_lp:     jmp   $F602
_puts:        jmp   $F60A
_gethexbyte:  jmp   $F60E
_get_atbl:    jmp   $F62E
_tohex:       jmp   $F641
_ishex:       jmp   $F64A
_clr_spaces:  jmp   $F660
_next_hex:    jmp   $F66C
_first_hex:   jmp   $F677
_gethex:      jmp   $F67E
_readhex:     jmp   $F686
_numck_lp:    jmp   $F689
_disasm:      jmp   $F6FA




