;******************************************************************************
;                   Routines to Support SEEPROM (Atmel AT25HP512)
;                      Written by Steve Holton N1NB  December 2004
;******************************************************************************

;Atmel AT25HP512 (SEEPROM) Equates

at25_WREN    equ     $06             ; Write enable
at25_WRDI    equ     $04             ; Reset Write enable latch (disable write)
at25_RDSR    equ     $05             ; Read Status register
at25_WRSR    equ     $01             ; Write Status register
at25_READ    equ     $03             ; Read
at25_WRITE   equ     $02             ; Write

; end equates ---------------------------------------------------------------

;------------------- SPI routines for Atmel AT25HP512 ---------------------


;----------------------------------------------------------------------------
;       initializes and enables SPI, sets /CS high (not selected)
SPI_Init:
        bclr    SPE,spcr        ; disable SPI module before configuring
        bset    SEE_CS,DDRE     ; set chip select as output
        mov     #00,spscr       ; modem divisor 2 (highest)
        bset    SEE_CS,portE    ; set CS high
        mov     #$20,spcr       ; set initial state Master+ CPOL&CPHA=0 (mode 0)
        bset    SPE,spcr        ; enable SPI module
        rts



;---------------------------------------------------------------------
;             sends write enable - must be used before any write (writepage & write enable)
;                                  destoys A
SPI_WriteEnable:
        bclr    SEE_CS,portE      ; set /CS low
        brclr   SPTE,spscr,*      ; Wait for transmitter empty
        mov     #at25_WREN,spdr   ; send Write enable
        brclr   SPTE,spscr,*      ; Wait for transmitter empty
        brclr   SPRF,spscr,*      ; Wait for "phony" received byte
        lda     spdr              ; clear the phony received data
        bset    SEE_CS,portE      ; set /CS high
        rts

;-------------------------------------------------------------------------
;              wait for SEEPROM to be ready returns status reg in A
SPI_DeviceReady:
        jsr     SPI_ReadStatusReg
        psha
        and     #$01              ;test for ready
        pula                      ; return status reg in a
        bne     SPI_DeviceReady
        rts

;--------------------------------------------------------------------
;               Performs 128 byte page write (only form supported by AT25HP512
;               H:X points to 2 words in memory;
;                   1st word is RAM address of data to be written
;                   2nd word is SEEPROM address were data will be written
;                       should be 128 byte aligned
;                       If not I think it may wrap OK
;               Upon completion:   H:X 1 byte beyond last byte written from RAM
;                                  A destroyed
SPI_WritePage:
        lda     ,x
        psha
        aix     #+1
        lda     ,x
        psha
        aix     #+1
        bclr    SEE_CS,portE      ; set CS low

        brclr   SPTE,spscr,*      ; Wait for transmitter empty
        mov     #at25_Write,spdr  ;  send Write
        brclr   SPTE,spscr,*      ; Wait for transmitter empty
        brclr   SPRF,spscr,*      ; Wait for "phony" received byte
        lda     spdr              ; clear the phony received data

        mov     x+,spdr           ; Hi order address bits
        brclr   SPTE,spscr,*      ; Wait for transmitter empty
        brclr   SPRF,spscr,*      ; Wait for "phony" received byte
        lda     spdr              ; clear the phony received data

        mov     x+,spdr           ; low order address bits
        brclr   SPTE,spscr,*      ; Wait for transmitter empty
        brclr   SPRF,spscr,*      ; Wait for "phony" received byte
        lda     spdr              ; clear the phony received data

        pulx                      ; pull lo byte into X
        pulh                      ; pull hi byte into H
        lda     #128              ; byte count = 1 page = 128 bytes
page_write_loop:
        mov     x+,spdr         ; move byte & increment ptr
        brclr   SPTE,spscr,*    ; Wait for transmitter empty
        brclr   SPRF,spscr,*    ; Wait for "phony" received byte
        psha
        lda     spdr            ; Clear 'phony' received data before exit
        pula
        deca
        bne     page_write_loop


        brset   SEE_SCK,portE,* ; wait for clock low (from Atmel FAQ) prrbably bogus for SPI
        bset    SEE_CS,portE    ; set CS high
        rts


;-----------------------------------------------------------
;               Read N bytes (N=0 read 256 bytes, 1<N<256 read N specified in A)
;               H:X points to 2 words in memory;
;                   1st word is RAM address of data were data is read into
;                   2nd word is SEEPROM address were data read from
;               Upon completion:   H:X 1 byte beyond last byte read into RAM
;                                  A destroyed

SPI_Read:
        psha                      ; save byte count
        lda     ,x                ; get ram address  Hi byte
        psha                      ; save on stack for later
        aix     #+1
        lda     ,x                ; ditto for lo byte
        psha
        aix     #+1
        bclr    SEE_CS,portE      ; set CS low
        brclr   SPTE,spscr,*      ; Wait for transmitter empty

        mov     #at25_Read,spdr   ; send Read command
        brclr   SPTE,spscr,*      ; Wait for transmitter empty
        brclr   SPRF,spscr,*      ; Wait for received data
        lda     spdr              ; clear 'phony' received data before actual data comes

        mov     x+,spdr           ; Hi order address bits
        brclr   SPTE,spscr,*      ; Wait for transmitter empty
        brclr   SPRF,spscr,*      ; Wait for received data
        lda     spdr              ; clear 'phony' received data before actual data comes

        mov     x+,spdr           ; low order address bits
        brclr   SPTE,spscr,*      ; Wait for transmitter empty
        brclr   SPRF,spscr,*      ; Wait for received data
        lda     spdr              ; clear 'phony' received data before actual data comes

        pulx                      ; pull lo byte into X
        pulh                      ; pull hi byte into H
        pula                      ; get byte count

byte_read_loop:
        mov     #00,spdr        ; send something to get something back
        brclr   SPTE,spscr,*    ; Wait for transmitter empty
        brclr   SPRF,spscr,*    ; Wait for received data
        mov     spdr,x+         ; move data byte & increment ptr
        deca
        bne     Byte_read_loop

        bset    SEE_CS,portE    ; set CS high
        rts

;---------------------------------------------------------------------------
;               reads SSEPROM status reg and returns it in A

SPI_ReadStatusReg:
        bclr    SEE_CS,portE      ; set CS low
        brclr   SPTE,spscr,*      ; Wait for transmitter empty
        mov     #at25_RDSR,spdr   ;  send Read status reg
        brclr   SPTE,spscr,*      ; Wait for transmitter empty
        brclr   SPRF,spscr,*      ; Wait for Received data
        lda     spdr              ; clear the SPRF status for 'phony' data
        mov     #00,spdr          ; send something to get something
        brclr   SPTE,spscr,*      ; Wait for transmitter empty
        brclr   SPRF,spscr,*      ; Wait for Received data
        lda     spdr              ; clear SPRF getting the actual data from SEEPROM status Reg
        bset    SEE_CS,portE      ; set /cs high
        rts

;
;---------------------------------------------------------------------------
;               Write SEEPROM status reg with data from A


SPI_WriteStatusReg:
        bclr    SEE_CS,portE      ; set CS low
        psha                      ; save Status byte
        mov     #at25_WRSR,spdr   ;  send Write status reg
        brclr   SPTE,spscr,*      ; Wait for transmitter empty
        brclr   SPRF,spscr,*      ; Wait for Received data
        lda     spdr              ; clear 'phony' data
        pula                      ; recover status byte
        sta     spdr              ; send status byte from A
        brclr   SPTE,spscr,*      ; Wait for transmitter empty
        brclr   SPRF,spscr,*      ; Wait for Received data
        lda     spdr              ; Clear phony received byte
        bset    SEE_CS,portE      ; set /cs high
        rts
